#ifndef APPLICATIONACTIONMEDIATOR_H
#define APPLICATIONACTIONMEDIATOR_H

#include <QObject>
#include <QSharedPointer>
#include "applicationactioncontent.h"

class MWidget;
class ApplicationInterface;

/*!
 * \class ApplicationActionMediator
 * \brief ApplicationActionMediator mediates actions between applications
 * and application extensions.
 */
class ApplicationActionMediator : public QObject
{
    Q_OBJECT

public:
    /*!
     * \brief Returns a mediator instance
     * \return A pointer to the mediator
     */
    static ApplicationActionMediator* instance();

    /*!
     * \brief Sends a view action to the extension
     * \param content the content of the view action
     */
    virtual void sendViewAction(ApplicationActionContent *content);

    /*!
     * \brief Sends a play action to the extension
     * \param content the content of the play action
     */
    virtual void sendPlayAction(ApplicationActionContent *content);

    /*!
     * \brief Opens a subpage in the application
     * \param widget the widget to show as the page content
     * \param title the title of the page
     */
    virtual void showSubPage(MWidget *widget, const QString &title);

    /*! \brief Starts the playback in the application
     * \param uriList List of the media Uris to play
     * \param playIndex The starting index for playback
     * \param callerId Optional identification for the caller
     */
    void play(const QStringList &uriList,
              uint playIndex = 0,
              const QString &callerId = QString());

    /*!
     * \brief Returns the last view action
     * \return a QSharedPointer to the last view action
     */
    virtual QSharedPointer<ApplicationActionContent> lastViewAction();

    /*!
     * \brief Returns the last play action
     * \return a QSharedPointer to the last play action
     */
    virtual QSharedPointer<ApplicationActionContent> lastPlayAction();

    /*!
     * \brief Returns the application interface
     * Returns the applicaation interface, which is used by the
     * application to listen to commands sent by extensions. Extensions
     * should never use it directly.
     * \return a pointer to the application interface
     */
    ApplicationInterface* applicationInterface() const;

    /*!
     * \brief Sets the application interface
     * Sets the application interface that is used by the application
     * to listen the commands sent by extensions. Extensions should never
     * use it directly.
     * \param interface a pointer to the application interface
     */
    void setApplicationInterface(ApplicationInterface *interface);

signals:
    /*!
     * \brief Signaled when a view has been opened
     * viewStarted is signaled any time a view is opened in the application.
     * The content parameter describes what kind of view action occured.
     * \param content a QSharedPointer to the action content.
     */
    void viewStarted( QSharedPointer<ApplicationActionContent> content );

    /*!
     * \brief Signaled when a media has been played
     * playStarted is signaled any time a media file is played in the application.
     * The content parameter describes the played file.
     * \param content a QSharedPointer to the action content.
     */
    void playStarted( QSharedPointer<ApplicationActionContent> content );

protected:
    ApplicationActionMediator(QObject *parent = 0);
    virtual ~ApplicationActionMediator();

    static ApplicationActionMediator *instanceToThis;

    QSharedPointer<ApplicationActionContent> viewAction;
    QSharedPointer<ApplicationActionContent> playAction;

    ApplicationInterface *appInterface;
};

#endif // APPLICATIONACTIONMEDIATOR_H
